require('muicss');
const $ = require('jquery');
const _ = require('underscore');
const Backbone = require('backbone');
require('backbone.modal');
const { TrainModel, TrainView } = require('./train');
const { RouteModel, RouteView } = require('./route');

const live = true;

if (live) {
  const url = new URL(window.location.origin);
  console.log('url', url);

  window.loc = url.origin;

  if ('serviceWorker' in navigator) {
    //
    navigator.serviceWorker.ready.then(function(reg) {
      console.warn('Ready??', reg);
      // main();
    });

    window.addEventListener('load', function() {
      navigator.serviceWorker
        .register('./service-worker.js')
        .then((r) => {
          console.warn('Service Worker Registered', r.scope);
        })
        .catch((error) => {
          // registration failed
          console.error(`Registration failed with ${ error}`);
        });
    });

    //
  }
}

(function () {
  let offline = false;
  const bridger = new Worker('bridger.js');
  const $snackbar = $('#snackbar');

  const app = {
    'routes' : [
      { 'from': 'dbe', 'to': 'glq' },
      { 'from': 'glq', 'to': 'dbe' },
      { 'from': 'pyg', 'to': 'glc' },
      { 'from': 'glc', 'to': 'ptk' },
      { 'from': 'ptk', 'to': 'dbe' },
      { 'from': 'glc', 'to': 'eus' },
      { 'from': 'eus', 'to': 'glc' },
      { 'from': 'pad', 'to': 'hxx' },
      { 'from': 'glc', 'to': 'bhi' },
      { 'from': 'bhi', 'to': 'glc' },
      { 'from': 'dbe', 'to': 'dmr' },
      { 'from': 'dmr', 'to': 'glc' }
    ],
    'views':{}

  };

  const Modal = Backbone.Modal.extend({
    'template': '#modal-template',
    'cancelEl': '.bbm-button',
    'content' : 'bob'
  });

  app.eventBus = _.extend({}, Backbone.Events);

  app.eventBus.on('offline', function() {
    if (offline === false) {
      offline = true;
      // console.log('Set it once!');
      $snackbar.addClass('show');
    }
  }, this);

  app.eventBus.on('online', function() {
    if (offline === true) {
      offline = false;
      // console.log('Set it once!');
      $snackbar.removeClass('show');
    }
  }, this);

  app.eventBus.on('showModal', function(d) {
    //  'template': '#modal-template',
    console.log('!! ShowModal');

    console.log('d', d);

    const obj = { 'content': d.d };
    const modalView = new Modal();

    console.log(modalView);
    const html = modalView.render().el;
    $('#_hero').html(html);
    const $modalContent = $('#modalContent');
    $modalContent.html(d.d);
  }, this);

  app.routeView = new RouteView( { 'model': new RouteModel({ 'bus': app.eventBus }), 'eventBus': app.eventBus, 'el': $('#trainResults') });

  app.createViews = function() {
    for (const route of this.routes) {
      // console.log(route);
      const key = Symbol(route.from + route.to);
      // console.log(key);
      route.bus = this.eventBus;
      this.views[key] = new TrainView({ 'model': new TrainModel(route), 'eventBus': app.eventBus });
    }
  };

  /* if ('serviceWorker' in navigator)
    navigator.serviceWorker
      .register('./service-worker.js')
      .then(function() {
        console.log('Service Worker Registered');
      });*/

  app.createViews();
})();
