const $ = require('jquery');
const _ = require('underscore');
const Backbone = require('backbone');
const { createPanel, addPanel } = require('./libs/panel');

function minuteFloor() {
  const now = new Date();

  return parseInt(~~(now.getTime() / 60000) * 60000, 10).toString(32);
}

const RouteModel = Backbone.Model.extend({
  'initialize': function () {
    const fromStation = this.get('from');
    const toStation = this.get('to');

    const routeUrl = `/gettrains?from=${ this.get('from') }&to=${ this.get('to')}`;
    const target = this.get('from') + this.get('to');
    this.set('url', routeUrl);
    this.set('routeUrl', routeUrl);
    this.set('target', target);
    this.set('visible', false);
    this.set('trainData', { 'eta':'OFF', 'sta': 'OFF' });
    this.set('interval', null);
    this.interval = null;
    this.update();
  },
  'update': function (m) {
    console.log('Route model update');
    console.log(this.changedAttributes());
    console.log(_.keys(this.changedAttributes()));
    if (!this.get('to'))
      return;

    const now = new Date;
    const hours = now.getHours();
    const limit = (hours < 6) ? 3600000 : 95000;

    const mod = limit - (now.getTime() % limit);
    console.log('mod', mod);

    if (hours >= 6)
      this.getRoute();
    else
      this.set('trainData', { 'eta':'OFF', 'sta': 'OFF' });

    const routeUpdateFn = function () {
      console.log('routeUpdateFn');
      this.update();
    };

    clearTimeout(this.interval);
    this.interval = 0;
    this.interval = setTimeout(routeUpdateFn.bind(this), mod + 10);
  },
  'getRoute': function () {
    const url = `/gettrains?from=${ this.get('from') }&to=${ this.get('to')}&m=${minuteFloor()}`;
    const self = this;

    $.ajax({
      'type': 'GET',
      'url': url,
      'data': '',
      'dataType': 'json',

      'timeout': 10000,
      'headers': {
        'Access-Control-Allow-Origin': '*',
        'Access-Control-Allow-Methods': 'PUT, GET, POST, DELETE, OPTIONS',
        'Access-Control-Allow-Headers': 'Content-Type'

      },
      'success': function (data) {
        // getTrainsCB(data);
        console.log('Got', data);

        self.set('route', data);
        self.set('visible', true);
      },
      'error': function (xhr, type) {
        console.error('ajax error');
        console.log('readyState', xhr.readyState);
        console.log('status', xhr.status);
        console.error(type);
      }
    });
  },
  'setRoute': function(from, to) {
    this.set('from', from);
    this.set('to', to);
  },
  'getNews': function(query) {
    console.log('model:getNews');
    const url = `/getnews?q=${ query}`;
    const self = this;
    const bus = this.get('bus');
    $.ajax({
      'type': 'GET',
      'url': url,
      'data': '',
      'dataType': 'json',

      'timeout': 10000,
      'headers': {
        'Access-Control-Allow-Origin': '*',
        'Access-Control-Allow-Methods': 'PUT, GET, POST, DELETE, OPTIONS',
        'Access-Control-Allow-Headers': 'Content-Type'

      },
      'success': function (data) {
        // getTrainsCB(data);
        console.log('Got', data);

        bus.trigger('showModal', data);
        // self.set('route', data);
        // self.set('visible', true);
      },
      'error': function (xhr, type) {
        console.error('ajax error');
        console.log('readyState', xhr.readyState);
        console.log('status', xhr.status);
        console.error(type);
      }
    });
  }
});

const RouteView = Backbone.View.extend({
  'tagName': 'div',

  'initialize': function (options) {
    _.bindAll(this, 'render', 'stations', 'news');
    this.eventBus = options.eventBus;
    // this.model.bind('change', this.render);
    /* this.model.bind('change:route', function(e){
       console.log('event:', e);
     });*/
    this.listenTo(this.model, 'change:route', _.debounce(_.bind(this.render), 128));
    this.$trains = $('#trains');
    this.$traininfo = $('#traininfo');
    this.$traintext = $('#trainResults');
    this.urlParse = /^(?:(http[s]?|ftp[s]):\/\/)?([^:\/\s]+)(:[0-9]+)?((?:\/\w+)*\/)([\w\-\.]+[^#?\s]+)([^#\s]*)?(#[\w\-]+)?$/;
    // this.$el = this.$traintext;
    // this.el = this.$traintext[0];
    console.log(this);
    this.eventBus.on('showRoute', function(d) {
      console.log('Showroute', d);
      this.showPanel();
      this.model.set('to', d.to);
      this.model.set('from', d.from);
      this.model.update();
    }, this);

    this.initView();
  },
  'showPanel': function(guid) {
    console.log('Showing panel', guid);

    this.$newPanel = createPanel({ 'title':'Details', 'divId':'routeP' });

    this.$el = addPanel(this.$newPanel);

    // this.$el.empty();
    this.$newPanel.show();
  },
  'events': {
    'click a': 'news',
    'click .station': 'stations'
  },
  'stations': function(e) {
    console.log('Stations', e);
    this.eventBus.trigger('showModal');
  },
  'news': function(e) {
    const target = e.currentTarget;
    const data = target.dataset;

    console.log('News', e);
    if (data.hasOwnProperty('url')) {
      const url = data.url;
      const query = this.urlParse.exec(url)[5];
      console.log(url);
      console.log(query);
      this.model.getNews(query);
    }
  },
  'render': function (m) {
    console.log('Routeview:Render');
    console.log('m', m);
    console.log('r', m.changedAttributes());
    console.log('r', _.keys(m.changedAttributes()));
    const route = this.model.get('route');

    console.log(route);

    const symbol = ['💠', '🚉'];
    let ws = '';
    const services = [];
    if (typeof route.trainServices === 'object' && route.trainServices !== null)
      for (const item of route.trainServices) {
        const dest = item.destination[0];
        const via = dest.via !== null ? ` <em class="mui--text-accent">${dest.via}</em>` : '';
        const platform = item.platform !== null ? item.platform : `${symbol[0]}`;
        // 🚉 💠
        const time = item.sta !== null ? item.sta : `<em class="mui--text-accent-secondary">D</em> ${item.std}`;
        const status = item.eta !== null ? item.eta : item.etd;
        const trainLength = item.length;

        const carriageCount = (trainLength > 0) ? ` (${trainLength} 🚃) ` : '';

        const statusMode = (status.toLowerCase() === 'on time') ? 'ontime' : 'delayed';
        const delayReason = (item.delayReason !== null) ? `<tr><td colspan="4" class="mui--bg-danger mui--text-white" style="font-size:75%;">${item.delayReason}</td></tr>` : '';

        const cancelReason = (item.cancelReason !== null) ? item.cancelReason : 'No reason given 🤷';

        services.push({ 'location':dest.locationName, 'time':time, 'status':status, 'platform':platform, 'cancel':cancelReason, 'type':'train' });
        if (!item.isCancelled)
          ws = `${ws }<tr><td data-id="${item.serviceIdUrlSafe}" class="station">${dest.locationName}${carriageCount}${via}</td>
                      <td class="mui--text-center time">${time}</td>
                      <td class="mui--text-center ${statusMode}">${status}</td>
                      <td class="mui--text-center">${platform}</td>
                      </tr>${delayReason}`;
        else
          ws = `${ws }<tr><td>${dest.locationName} ${via}</td><td>${time}</td>
                      <td colspan="2" class="delayed">❌ ${cancelReason}</td></tr>`;
      }

    if (typeof route.busServices === 'object' && route.busServices !== null)
      for (const item of route.busServices) {
        const dest = item.destination[0];
        const via = dest.via !== null ? `<em>${dest.via}</em>` : '';
        const platform = item.platform !== null ? item.platform : '';
        const time = item.sta !== null ? item.sta : `D ${item.std}`;
        const status = item.eta !== null ? item.eta : item.etd;
        services.push({ 'location':dest.locationName, 'time':time, 'status':status, 'platform':platform, 'cancel':item.cancelReason, 'type':'bus' });

        ws = `${ws }<tr><td>🚌 ${dest.locationName} ${via}</td><td>${time}</td><td>${status}</td><td>${platform}</td></tr>`;
      }

    let nrMessages = '';
    if (typeof route.nrccMessages === 'object' && route.nrccMessages !== null)
      // we have national rail messages so put a box at the top
      // <div class="mui--bg-danger .mui--text-white" style="height:10px;"></div>
      for (const item of route.nrccMessages) {
        const msg = item.value.replace('href=', 'data-url=').replace(' ">', '">');

        nrMessages = `${nrMessages}<div class="mui--bg-danger mui--text-white nrccAlert" style="padding:2px;">${msg}</div>`;
      }

    const thead = `<div class="mui--text-center mui--text-accent">${route.locationName} TO ${route.filterLocationName}</div>
${nrMessages}
                <table class="mui-table mui-table-bordered">
                <thead><tr><th>Destination</th>
                                <th class="mui--text-center">Time</th>
                                <th class="mui--text-center">Status</th>
                                <th class="mui--text-center">Platform</th></tr></thead>`;

    ws = `${thead}<tbody class="tableBody">${ws}</tbody></table>`;
    //  ws = `${thead}${ws}</table>`;

    this.$el.empty().html(ws);
    // this.$traintext.removeClass('mui--hide').addClass('mui--show');

    /*
    if (this.$trains.hasClass('unsliced'))
      this.$trains.addClass('sliced').removeClass('unsliced');
*/

    // this.$trains.css("background-color", "yellow");
    // this.$trains.css("height", "300px");

    this.$el.find('a').on('click', this.news);
  },
  'initView': function () {

  },
  'showTrains': function () {
    console.log('Show train');
  },
  'viewRoute': function(from, to) {
    console.log('View route', from, to);
  }

});

module.exports = { RouteModel, RouteView };
