/**
 *
 * User: Martin Donnelly
 * Date: 2016-10-03
 * Time: 14:20
 *
 */
const $ = require('jquery');
const _ = require('underscore');
const Backbone = require('backbone');
const { findStation } = require('./stations');

function hourFloor() {
  const now = new Date();

  return parseInt(~~(now.getTime() / 3600000) * 3600000, 10).toString(32);
}

function minuteFloor() {
  const now = new Date();

  return parseInt(~~(now.getTime() / 60000) * 60000, 10).toString(32);
}

const TrainModel = Backbone.Model.extend({
  'defaults': function (obj) {
    // return a new object
    return {
      'update': 0,
      'last': new Date().getTime()
    };
  },
  'initialize': function () {
    const fromStation = this.get('from');
    const toStation = this.get('to');
    const url = `/getnexttraintimes?from=${ this.get('from') }&to=${ this.get('to')}`;
    const routeUrl = `/gettrains?from=${ this.get('from') }&to=${ this.get('to')}`;
    const target = this.get('from') + this.get('to');
    const fancyTarget = `${findStation(this.get('from'))}<br/>${findStation(this.get('to'))}`;
    this.set('url', url);
    this.set('routeUrl', routeUrl);
    this.set('target', target);
    this.set('fancyTarget', fancyTarget);
    this.set('visible', false);
    this.set('trainData', { 'eta':'OFF', 'sta': 'OFF' });
    this.update();
  },
  'update': function () {
    const now = new Date;
    const hours = now.getHours();
    const limit = (hours < 6) ? 3600000 : 60000;

    const mod = limit - (now.getTime() % limit);

    if (hours >= 6)
      this.getTrain();
    else
      this.set('trainData', { 'eta':'OFF', 'sta': 'OFF' });

    const trainUpdateFn = function () {
      this.update();
    };

    setTimeout(trainUpdateFn.bind(this), mod + 10);
  },
  'getTrain': function () {
    this.set('update', new Date().getTime());
    const url = this.get('url');
    const self = this;
    const bus = this.get('bus');
    $.ajax({
      'type': 'GET',
      'url': `${url}&m=${minuteFloor()}`,
      'data': '',
      'dataType': 'json',
      'timeout': 10000,
      'headers': {
        'Access-Control-Allow-Origin': '*',
        'Access-Control-Allow-Methods': 'PUT, GET, POST, DELETE, OPTIONS',
        'Access-Control-Allow-Headers': 'Content-Type'
      },
      'success': function (data) {
        bus.trigger('online');

        console.log('Data', data);
        self.set('trainData', data);
      },
      'error': function (xhr, type) {
        if (xhr.readyState === 0 && xhr.status === 0)

          bus.trigger('offline');
      }
    });
  }
});

const TrainView = Backbone.View.extend({
  'tagName': 'div',
  'initialize': function (options) {
    _.bindAll(this, 'render');

    this.eventBus = options.eventBus;
    this.model.bind('change:trainData', this.render);
    this.model.bind('change:update', function(){
      console.log('Updating....');
    });
    this.model.bind('all', function(eventName) {
      console.log(`${eventName } was triggered!`);
    });
    this.$trains = $('#trains');
    this.$traininfo = $('#traininfo');
    this.$traintext = $('#trainResults');
    this.$el = this.$trains;
    this.initView();
    //  console.log(this.get('routeView'));
    // console.log(this);
  },
  'events': {
    'click': 'showTrains'
  },
  'render': function () {
    const obj = this.model.get('trainData');
    const visible = this.model.get('visible');
    const route = this.model.get('route');

    const output = (obj.eta.toLowerCase() === 'on time') ? obj.sta : obj.eta;
    const status = (obj.eta.toLowerCase() === 'on time') ? 'ontime' : 'delayed';

    this.$button.html(output);
    this.$button.removeClass('delayed').removeClass('ontime').addClass(status);
  },
  'initView': function () {
    const self = this;
    const target = this.model.get('target');
    const fancyTarget = this.model.get('fancyTarget');
    const html = `
<div class="mui-row card">
<div class='mui-col-xs-7 mui-col-md-7 entry'>${fancyTarget}</div>
 <div class='mui-col-xs-5 mui-col-md-5 mui--text-right'> 
 <button class="mui-btn mui-btn--flat time" id="${target}"></button>
 </div>
</div>

`;
    this.$html = $(html);
    this.$html.on('click', () => {
      // console.log('from', self.model.get('from'));
      // console.log('to', self.model.get('to'));
      self.eventBus.trigger('showRoute', { 'from': self.model.get('from'), 'to': self.model.get('to') });
      //            console.log(self)
      // self.model.getRoute();
    });
    this.$trains.append(this.$html);

    this.$button = $(`#${target}`);

    const output = '<i class="fa fa-refresh mui--align-middle" ></i>';
    const status = (output === 'on time') ? 'ontime' : 'delayed';

    this.$button.html(output);
    this.$button.removeClass('delayed').removeClass('ontime').addClass(status);

    const cevent = `click #${target}`;
    this.events[cevent] = 'showTrains';
  },
  'showTrains': function () {
    // console.log('Show train');
    this.eventBus.trigger('showRoute');
  }

});

module.exports = { TrainModel, TrainView };
