// Copyright 2016 Google Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
const CACHE_VERSION = { 'version': '0.0.21' };
const PRECACHE = `traintimesData-v${CACHE_VERSION.version}`;
const cacheName = `traintimePWA-final-${CACHE_VERSION.version}`;
const RUNTIME = 'runtime';

const PRECACHE_URLS = [
  '/',
  '/index.html',
  '/service-worker.js',
  '/site.webmanifest',
  '/browserconfig.xml',
  '/css/style.min.css',
  '/css/mui.custom.css',
  '/fonts/fonts.css',
  '/fonts/gotham.css',
  '/fonts/Roboto-normal-400.woff',
  '/fonts/Roboto_Mono-normal-400.woff',
  '/js/bundle.js',
  '/js/vendor.js',
  '/img/Icon-36.png',
  '/img/Icon-48.png',
  '/img/Icon-72.png',
  '/img/Icon-96.png',
  '/img/Icon-144.png',
  '/img/Icon-192.png',
  '/img/Icon-512.png'
];

self.addEventListener('install', event => {
  console.warn('Installing...');
  event.waitUntil(
    caches.open(PRECACHE)
      .then(cache => cache.addAll(PRECACHE_URLS))
      .then(self.skipWaiting())
  );
});

// The activate handler takes care of cleaning up old caches.
self.addEventListener('activate', event => {
  console.warn('Activate...');
  const currentCaches = [PRECACHE, RUNTIME];
  event.waitUntil(
    caches.keys().then(cacheNames => {
      return cacheNames.filter(cacheName => !currentCaches.includes(cacheName));
    }).then(cachesToDelete => {
      return Promise.all(cachesToDelete.map(cacheToDelete => {
        return caches.delete(cacheToDelete);
      }));
    }).then(() => self.clients.claim())
  );
});

// The fetch handler serves responses for same-origin resources from a cache.
// If no response is found, it populates the runtime cache with the response
// from the network before returning it to the page.
self.addEventListener('fetch', event => {
  console.warn('Fetch', event.request.url);
  // Skip cross-origin requests, like those for Google Analytics.
  /* if (event.request.url.startsWith(self.location.origin)) {
    console.log('One of our requests..');
    event.respondWith(
      caches.match(event.request).then(cachedResponse => {
        if (cachedResponse) {
          console.log('cachedResponse', cachedResponse);
          return cachedResponse;
        }

        return caches.open(RUNTIME).then(cache => {
          return fetch(event.request).then(response => {
            // Put a copy of the response in the runtime cache.
            return cache.put(event.request, response.clone()).then(() => {
              console.log('fetch cache response', response);
              return response;
            });
          });
        });
      })
    );
  }*/

  if (event.request.url.startsWith(self.location.origin)) {
    console.log('!!', event.request);

    /* event.respondWith(
      caches.open(RUNTIME).then(function(cache) {
        return cache.match(event.request).then(function (response) {
          console.log('£', response);

          return response || fetch(event.request).then(function(response) {
            cache.put(event.request, response.clone());

            return response;
          });
        });
      })
    );*/

    event.respondWith(
      caches.open(RUNTIME).then(function(cache) {
        return cache.match(event.request).then(function (response) {
          return response || fetch(event.request).then(function(response) {
            cache.put(event.request, response.clone());

            return response;
          });
        });
      })
    );
  }

  if (event.request.url.startsWith('https://maps.googleapis.com')) {
    const url = new URL(event.request.url);
    const locCache = `loc-${url.searchParams.get('latlng')}`;
    event.respondWith(
      caches.match(locCache).then(cachedResponse => {
        if (cachedResponse)
          return cachedResponse;

        return caches.open(RUNTIME).then(cache => {
          return fetch(event.request).then(response => {
            // Put a copy of the response in the runtime cache.
            return cache.put(locCache, response.clone()).then(() => {
              return response;
            });
          });
        });
      })
    );
  }
});
